/* 
 * mainanim.c - This file contains an animation driver for the raytracer.
 *
 *  $Id: mainanim.c,v 1.11 1998/08/08 20:09:53 johns Exp $
 */

#ifdef MPI_DUMMY  /* EBCEEB */
#  include <mpi.h>
#endif
#include <sys/time.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "api.h"

#if defined(MPI_DUMMY)
# define rt_mynode() commRank
#else
  int rt_mynode(void); /* proto */
#endif

#define MAXFRAMES 100
#define MAXFRAMES_HI 1000
#define LOOP 100.0
#define LOOP2 50.0
#define RAD 6.28

#ifdef cube
#define RFILE "/cfs/johns/anim/frame"
#else
#define RFILE "/tmp/outfile"
#endif

char *path_prefix = RFILE;

void usage(char *error)
{
  if(error)
    printf("ERROR: %s\n", error );
  printf(
    "Usage: animray [-v] [-t] [-n NN] [-o PATH_PREFIX]\n"
    "  -o  Produce files as PATH_PREFIX.dddd.tga (default = \"%s\")\n"
    "  -v  Verbose = report for each frame\n"
    "  -t  Timing = show elapsed time\n"
    "  -n  Produce NN frames (default = %d)\n",
    path_prefix, MAXFRAMES );
}

int main(int argc, char **argv) {
  SceneHandle scene;
  int i,j;
  apivector ctr, ctr1, vect2;
  apitexture tex1, tex2;
  apiflt f1, f2;
  char fname[200];
  char fname2[200];

  int verboseFlag=0, timingFlag=0;
  int nframes = MAXFRAMES;
  struct timeval tv0, tv;

#ifdef MPI_DUMMY  /* EBCEEB */
  int commRank, commSize;
  MPI_Init( &argc, &argv );
  MPI_Comm_rank( MPI_COMM_WORLD, &commRank );
  MPI_Comm_size( MPI_COMM_WORLD, &commSize );
#endif

  rt_initialize(&argc, &argv); 

  for( i=1; i < argc; i++ )
  {
    if( !strcmp(argv[i],"--help") )
      { usage(NULL); goto done; }
    else if( !strcmp(argv[i],"-v") )
      verboseFlag = 1;
    else if( !strcmp(argv[i],"-t") )
      timingFlag = 1;
    else if( !strcmp(argv[i],"-n") )
    {
      if( ++i >= argc )
        { usage("missing number after \"-n\""); goto done; }
      if( sscanf( argv[i], "%d", &nframes ) != 1 )
        { usage("not-numeric value after \"-n\""); goto done; }
      if( nframes <= 0 || nframes > MAXFRAMES_HI )
        { usage("too big or negative value after \"-n\""); goto done; }
      if (rt_mynode()==0)
        printf("Generate %d frames\n", nframes );
    }
    else if( !strcmp(argv[i],"-o") )
    {
      if( ++i >= argc )
        { usage("missing PATH_PREFIX after \"-o\""); goto done; }
      path_prefix = argv[i];
#if 0 /* check that path is valid : not tested */
      {
        char c, *p = strrchr( path_prefix, '/' );
        if( p )
          { c = *p; *p = 0; }
        if( access( path_prefix, W_OK|X_OK ) )
          { usage("invalid dir in PATH_PREFIX"); goto done; }
        if( p )
          *p = c;
      }
#endif
    }
  }

  tex1.col.r=1.0;
  tex1.col.g=1.0;
  tex1.col.b=1.0;
  ctr1.x=5.0;
  ctr1.y=5.0;
  ctr1.z=1.0;
  f1=0.5;

  tex2.col.r=1.0;
  tex2.col.g=1.0;
  tex2.col.b=1.0;
  tex2.ambient=0.1;
  tex2.diffuse=0.7;
  tex2.specular=0.2;
  tex2.opacity=1.0;
  tex2.texturefunc=0;
  f2=1.5;

  vect2.x=0.0;
  vect2.y=1.0;
  vect2.z=0.0;

  if( timingFlag )
      gettimeofday( &tv0, NULL );

#ifdef MPI_DUMMY  /* EBCEEB */
  for (i=commRank; i < nframes; i+=commSize) {
#else
  for (i=0; i < nframes; i++) {
#endif
    void * txarray[20];

    scene = rt_newscene();
    rt_resolution(scene, 320, 240);

    if( verboseFlag )
    {
      char timingStr[32];
      if( timingFlag )
      {
        gettimeofday( &tv, NULL );
        sprintf( timingStr, ", time = %ld secs", tv.tv_sec - tv0.tv_sec );
      } else
       timingStr[0] = 0;
#if defined(MPI_DUMMY)  /* EBCEEB */
      printf("Node %d: Rendering frame %d %s\n", commRank, i, timingStr );
#elif defined(MPI) || defined(cube)
      if (rt_mynode()==0) printf("Rendering frame %d %s\n",i,timingStr);
#else
      printf("Rendering frame %d %s\n",i,timingStr);
#endif
    } /* verboseFlag */

    ctr1.x= i / 20.0 - 5.0;
    ctr1.y=5.0;
    ctr1.z= i / 20.0 + 5.0; 
    tex1.col.r=1.0;
    tex1.col.g=1.0;
    tex1.col.b=1.0;
    txarray[0] = rt_texture(&tex1);
    rt_light(scene, txarray[0], ctr1, f1);

    ctr1.x= i / 15.0 - 5.0; 
    ctr1.y=0.0;
    ctr1.z=15.0;
    tex1.col.r=0.0;
    tex1.col.g=0.0;
    tex1.col.b=1.0;
    txarray[1] = rt_texture(&tex1);
    rt_light(scene, txarray[1], ctr1, f1 / 3.0 );

    j=i;
    tex2.col.r=1.0;
    tex2.texturefunc=0;
    ctr.x=2.0*sin((j*RAD) / LOOP);
    ctr.z=10.0 + 2.0*cos((j*RAD) / LOOP);
    ctr.y=-2.0;

    txarray[2] = rt_texture(&tex2);
    rt_sphere(scene, txarray[2], ctr, f2);

    tex2.texturefunc=0;
    j=i+(nframes/3);
    tex2.col.r=0.5 + 0.5*sin((j*RAD)/ LOOP2);
    ctr.x=2.0*sin((j*RAD) / LOOP);
    ctr.z=10.0 + 2.0*cos((j*RAD) / LOOP);
    ctr.y=-2.0;

    txarray[3] = rt_texture(&tex2);
    rt_sphere(scene, txarray[3], ctr, f2); 

    j=i+((nframes * 2)/3);
    tex2.col.r=1.0;
    tex2.col.b=0.5 + 0.5*sin((j*RAD)/ LOOP2);
    ctr.x=2.0*sin((j*RAD) / LOOP);
    ctr.z=10.0 + 2.0*cos((j*RAD) / LOOP);
    ctr.y=-2.0;

    txarray[4] = rt_texture(&tex2);
    rt_sphere(scene, txarray[4], ctr, f2);

    tex2.col.r=0.0;
    tex2.col.b=1.0;
    tex2.ambient=0.0;
    tex2.diffuse=0.0;
    tex2.specular=0.99;

    ctr.x=0.0;
    ctr.y=-2.0;
    ctr.z=10.0;

    txarray[5] = rt_texture(&tex2);
    rt_sphere(scene, txarray[5], ctr, f2); 

    tex2.diffuse=0.8;
    tex2.ambient=0.1;
    tex2.specular=0.2;
    tex2.col.r=1.0;
    ctr.y=-7.0;  

    txarray[6] = rt_texture(&tex2);
    rt_plane(scene, txarray[6], ctr, vect2); 

    sprintf(fname,".%4.4d.tga",i);
    strcpy(fname2, path_prefix);
    strcat(fname2, fname);

    rt_outputfile(scene, fname2);

    rt_renderscene(scene);

    rt_deletescene(scene);

    for (j=0; j<7; j++)
      free(txarray[j]);
  }

  if( timingFlag )
  {
# if defined(MPI_DUMMY)
    MPI_Barrier( MPI_COMM_WORLD );
# endif
# if defined(MPI_DUMMY) || defined(MPI) || defined(cube)
    if (rt_mynode()==0) 
# endif
    {
      gettimeofday( &tv, NULL );
      printf("%d frames. Elapsed %ld secs\n", nframes, tv.tv_sec-tv0.tv_sec);
    }
  }

done:
  rt_finalize();
#ifdef MPI_DUMMY  /* EBCEEB */
  MPI_Finalize();
#endif

  return 0;
}
   
